/**
  *  \file client/dialogs/passwordchange.cc
  *  \brief Password Change Dialog
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/dialogs/passwordchange.h"
#include "ui/window.h"
#include "ui/widgets/richstatic.h"
#include "ui/richparser.h"
#include "ui/widgets/inputline.h"
#include "util/nls.h"
#include "gfx/palette.h"
#include "ui/layout.h"
#include "ui/widgets/button.h"
#include "game/gen.h"
#include "ui/widgets/framegroup.h"
#include "ui/widgets/quit.h"

/** Edit the result file password.
    \param gen [out] Password stored here */
void
doPasswordChange(GGen& gen)
{
    // VBox
    //   UIMultilineStatic
    //   UIInputLine
    //   HBox
    //     "OK"
    //     "Cancel"
    //     "No password"
    const int WIDTH = 400;
    enum {
        btnOK         = 1000,
        btnCancel     = 1001,
        btnNoPassword = 1002
    };

    ObjectHolder h;
    UIWindow& window = h.add(new UIWindow(_("Change password"), UIWindow::BLUE_WINDOW, COLOR_GRAY));
    UIInputLine& input = h.add(new UIInputLine(10, WIDTH, UIInputLine::ilf_Hidden | UIInputLine::ilf_NoHi, 1000));
    UIGroup& group3 = h.add(new UIGroup(UIHBoxLayout::instance5));

    window.setLayout(UIVBoxLayout::instance5);

    window.add(h.add(new UIRichStatic(RichParser::textFromXml(_("<big>Enter new password:</big>\n"
                                                                "Remember that a result file password does not protect "
                                                                "against a determined attacker.")),
                                      WIDTH)));
    window.add(UIFrameGroup::wrap(h, input));
    window.add(group3);

    group3.add(h.add(new UIButton(btnOK, _("OK"),         SDLK_RETURN)).addStop());
    group3.add(h.add(new UIButton(btnCancel, _("Cancel"), SDLK_ESCAPE)).addStop());
    group3.add(h.add(new UIButton(btnNoPassword, _("Alt-D - No password"), ss_Alt + 'd')).addStop());
    group3.add(h.add(new UISpacer()));

    window.add(h.add(new UIQuit(btnCancel)));
    window.pack();
    input.setFocusedWidget();
    ui_root->centerWidget(window);
    int result = ui_root->runModal(window);

    switch (result) {
     case btnOK:
        gen.setPassword(input.getText());
        break;
     case btnCancel:
        break;
     case btnNoPassword:
        gen.setPassword(gen.NOPASSWORD);
        break;
    }
}
