/**
  *  \file client/screens/mkturnscreen.cc
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/screens/mkturnscreen.h"
#include "client/dialogs/consolelog.h"
#include "cpluslib/format.h"
#include "game/gamedirs.h"
#include "game/maketurn.h"
#include "game/racename.h"
#include "game/storage/overview.h"
#include "ui/msgbox.h"
#include "util/console.h"
#include "util/except.h"
#include "util/nls.h"



/** Generate turn files for all files in game_file_dir. */
void
doMaketurn()
{
    // Figure out what to do
    int nplayers = 0;
    GDirectoryOverview ov;
    ov.scan(*game_file_dir);
    for (int i = 1; i <= NUM_PLAYERS; ++i)
        if ((ov.getPlayerFlags(i) & GDirectoryOverview::have_Gamedir) != 0)
            ++nplayers;

    // Stupid case
    if (nplayers == 0)
        return;
    
    // Build a window
    WConsoleLogWindow log(_("Maketurn"), nplayers+1);
    log.scrollAfterEnd();
    log.open();

    // Do it
    try {
        GMaketurn mt(*game_file_dir);
        for (int i = 1; i <= NUM_PLAYERS; ++i) {
            if ((ov.getPlayerFlags(i) & GDirectoryOverview::have_Gamedir) != 0) {
                int ncommands = mt.makeTurn(i);
                ::console.write(LOG_INFO, format(_("%s:\t %d command%!1{s%}"),
                                                 player_racenames.getFullName(i),
                                                 ncommands),
                                ConsoleCentered);
            }
        }
        mt.writeTurns();

        console.write(LOG_INFO, _("Turn files have been compiled."));
        log.waitForConfirmation();
    }
    catch (FileProblemException& e) {
        messageBox(format(_("Turn files could not be compiled: file %s, %s"),
                          e.getFileName(),
                          e.what()),
                   _("Maketurn"));
    }
    catch (std::exception& e) {
        messageBox(format(_("Turn files could not be compiled: %s"), e.what()),
                   _("Maketurn"));
    }
}
