/**
  *  \file client/widgets/ammolist.cc
  *  \brief WAmmoList widget
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/widgets/ammolist.h"
#include "game/actions/baseammo.h"
#include "ui/uiglobal.h"
#include "game/spec.h"
#include "util/nls.h"
#include "util/unicodechars.h"
#include "game/reg.h"

/** Constructor.
    \param p Ammo building action */
WAmmoList::WAmmoList(GStarbaseBuildAmmoAction& p)
    : UIListbox(0, NUM_TORPS+1, 18*font_ems[FONT_NORMAL], NUM_TORPS+1),
      p(p)
{ }

WAmmoList::~WAmmoList()
{ }

bool
WAmmoList::isAccessible(int what)
{
    return what <= NUM_TORPS && p.canHave(what+1);
}

void
WAmmoList::drawPart(GfxCanvas& can, int from, int to)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useFont(*fonts[FONT_NORMAL]);
    for (int i = from; i < to; ++i) {
        const int n = getTop() + i + 1;
        GfxRect area;
        prepareEntry(ctx, ColorStyle, i, &area);
        if (n <= NUM_TORPS+1) {
            /* 1..NUM_TORPS are torpedoes, NUM_TORPS+1 is fighters,
               see GStarbaseBuildAmmoAction. */
            UIColor::Color c;
            if (int t = p.getTechUpgradeNeeded(n)) {
                if (t > 6 && !allowRegistered(p.getPlanet().getOwner())) {
                    c = UIColor::tc_Faded;
                } else {
                    c = UIColor::tc_Blue;
                }
            } else {
                c = UIColor::tc_Static;
            }
            if (!p.canHave(n)) {
                c = UIColor::tc_Faded;
            }
            ctx.setColor(c);

            /* Figure out text */
            string_t what;
            if (p.getAmount(n)) {
                what += UTF_BULLET;
            } else {
                what += " ";
            }
            if (n <= NUM_TORPS) {
                what += getTorp(n).getName();
            } else {
                what += _("Fighter");
            }
            outTextF(ctx, area, what);
        }
    }
}
