/**
  *  \file game/save.h
  *  \brief Saving Game Data
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_SAVE_H
#define PCC_V2_GAME_SAVE_H

#include "cpluslib/types.h"
#include "game/playerset.h"

class GGameTurn;
class GUniverse;
class GOutbox;
class Stream;
class Directory;

enum {
    /* Flags for saving */
    save_ChecksumCount = 1
};

void saveShips(const GUniverse& univ, Stream& str, const GPlayerSet src, uint32_t& checksum, const int flags);
void savePlanets(const GUniverse& univ, Stream& str, const GPlayerSet src, uint32_t& checksum, const int flags);
void saveBases(const GUniverse& univ, Stream& str, const GPlayerSet src, uint32_t& checksum, const int flags);

void saveOutbox35(GOutbox& outbox, Directory& dir, int player);
void saveOutbox(GOutbox& outbox, Directory& dir, int player);

uint32_t computeShipChecksum(const GUniverse& univ, const GPlayerSet src, const int flags);
uint32_t computePlanetChecksum(const GUniverse& univ, const GPlayerSet src, const int flags);
uint32_t computeBaseChecksum(const GUniverse& univ, const GPlayerSet src, const int flags);

void saveDirectory(GGameTurn& trn, Directory& dir, GPlayerSet player_mask);
void saveTurns(GGameTurn& trn, Directory& dir, GPlayerSet player_mask);

#endif
