#!/usr/bin/perl -w
#
#  Simple RLE compression
#
#  This is the RLE compression scheme used in PCC 1.x's resource file
#  and implemented in io/rle.cc.
#
use strict;

binmode STDIN;
binmode STDOUT;

my $numBlocks = 0;
my $totalBytes = 0;

print STDOUT pack "V", 0;
while (1) {
    # Read
    my $data;
    read (STDIN, $data, 10240) or last;
    my @bytes = unpack "C*", $data;

    # Population statistic
    my @pop = (0) x 256;
    foreach (@bytes) { ++$pop[$_] }

    # Find least frequent character
    my $prefix = 0;
    foreach (1 .. 255) { if ($pop[$_] < $pop[$prefix]) { $prefix = $_ } }

    # Block header
    print pack "vC", length($data), $prefix;

    # Block data
    my $runChar = 0;
    my $runLength = 0;
    foreach (@bytes) {
        if ($runChar != $_ || $runLength >= 255) {
            if ($runLength > 2 || $runChar == $prefix) {
                print pack "C*", $prefix, $runLength, $runChar;
            } else {
                print pack "C*", ($runChar) x $runLength;
            }
            $runChar = $_;
            $runLength = 1;
        } else {
            ++$runLength;
        }
    }

    # Final run
    if ($runLength > 0) {
        if ($runLength > 2 || $runChar == $prefix) {
            print pack "C*", $prefix, $runLength, $runChar;
        } else {
            print pack "C*", ($runChar) x $runLength;
        }
    }

    # Statistic
    ++$numBlocks;
    $totalBytes += length($data);
}

# Write total size
seek STDOUT, 0, 0;
print STDOUT pack "V", $totalBytes;
