%
%  PCC2 Interpreter: Optimizer Tests
%
%  This defines a gazillion subroutines of names "in.XX" and "out.XX".
%  When the "in.XX" is run through the optimizer, the result should be
%  the "out.XX" routine. This can be done automatically using the
%  test_opt.pl script, which assembles this thing and invokes
%  'teststmt' to test all these routines.
%
%  The routines are not executable by themselves.
%
%  Note that instruction fusion must be honored here, and c2asm.pl
%  therefore allows assembling fused instructions, e.g. pushlit(u)
%  for a pushlit fused with an unary operation.
%

.jumps symbolic


%%
%%  StoreDrop
%%

sub in.storeDrop1
  .local a
  storeloc a
  drop 1
endsub
sub out.storeDrop1
  .local a
  poploc a
endsub


sub in.storeDrop2
  .local a
  storeloc a
  drop 2
endsub
sub out.storeDrop2
  .local a
  poploc a
  drop 1
endsub


sub in.storeDrop3
  % do not combine this to an (invalid) 'drop -1'
  .local a
  storeloc a
  drop 0
endsub
sub out.storeDrop3
  .local a
  storeloc a
endsub


sub in.storeDrop4
  .local a
  storeloc a
  drop 0
  drop 1
endsub
sub out.storeDrop4
  .local a
  poploc a
endsub


%%
%%  MergeDrop
%%

sub in.mergeDrop1
  drop 1
  drop 1
  drop 1
endsub
sub out.mergeDrop1
  drop 3
endsub


sub in.mergeDrop2
  drop 1
  drop 0
  drop 0
  drop 2
endsub
sub out.mergeDrop2
  drop 3
endsub


%%
%%  NullOp
%%

sub in.nullOp1
  ssuspend
  drop 0
  ssuspend
endsub
sub out.nullOp1
  ssuspend
  ssuspend
endsub


sub in.nullOp2
  ssuspend
  swap 0
  ssuspend
endsub
sub out.nullOp2
  ssuspend
  ssuspend
endsub


sub in.nullOp3
  ssuspend
  dup 0    % Not a null operation!
  ssuspend
endsub
sub out.nullOp3
  ssuspend
  dup 0
  ssuspend
endsub


sub in.nullOp4
  ssuspend
  swap 1   % Not a null operation!
  ssuspend
endsub
sub out.nullOp4
  ssuspend
  swap 1
  ssuspend
endsub


%%
%%  EraseUnusedLabels
%%

sub in.eraseUnusedLabels1
  .num_labels 2
  jt #1
  label #0
  uinc
  label #1
  udec
endsub
.jumps absolute
sub out.eraseUnusedLabels1
  .num_labels 2
  jt l1
  uinc
l1:
  udec
endsub
.jumps symbolic


sub in.eraseUnusedLabels2
  % The label would normally break the 'pushint 1/badd' pattern, so we
  % see that it has been removed because the pattern has been applied.
  .num_labels 2
  jt #1
  pushint 1
  label #0
  badd
  label #1
  udec
endsub
.jumps absolute
sub out.eraseUnusedLabels2
  .num_labels 2
  jt l1
  uinc
l1:
  udec
endsub
.jumps symbolic


%%
%%  InvertJumps
%%
%%  The pattern is
%%     jump label
%%     jump anywhere
%%   label:
%%

sub in.InvertJumps1
  % Two jumps with identical condition
  .num_labels 2
  j #0
  j #1
  label #0
  uinc
  label #1
endsub
.jumps abs
sub out.InvertJumps1
  .num_labels 2
  uinc
endsub
.jumps sym


sub in.InvertJumps2
  % Two jumps with pop; the second pop causes this optimisation to not be applied.
  .num_labels 2
  jtp #0
  jfep #1
  label #0
  uinc
  label #1
endsub
.jumps abs
sub out.InvertJumps2
  .num_labels 2
  jtp l0
  jfep l1
l0:
  uinc
l1:
endsub
.jumps sym


sub in.InvertJumps3
  % Two jumps with opposite condition (regular inversion case)
  .num_labels 2
  jt #0
  jfe #1
  label #0
  uinc
  label #1
endsub
.jumps abs
sub out.InvertJumps3
  .num_labels 2
  jfe l1
  uinc
l1:
endsub
.jumps sym

sub in.InvertJumps3p
  % Two jumps with opposite condition (regular inversion case) - not applied due to pop (#328)
  .num_labels 2
  jtp #0
  jfe #1
  label #0
  uinc
  label #1
endsub
.jumps abs
sub out.InvertJumps3p
  .num_labels 2
  jtp l0
  jfe l1
l0:
  uinc
l1:
endsub
.jumps sym


sub in.InvertJumps4
  % Second jump never taken
  .num_labels 2
  jtf #0
  jt #1
  label #0
  uinc
  label #1
endsub
.jumps abs
sub out.InvertJumps4
  .num_labels 2
  uinc
endsub
.jumps sym

sub in.InvertJumps4a
  % Second jump never taken, jump degenerates into 'drop' - not applied due to pop (#328)
  .num_labels 2
  jtfp #0
  jt #1
  label #0
  uinc
  label #1
endsub
.jumps abs
sub out.InvertJumps4a
  .num_labels 2
  jtfp l0
  jt l1
l0:
  uinc
l1:
endsub
.jumps sym


sub in.InvertJumps5
  % Conditional followed by unconditional (common case)
  .num_labels 2
  jtp #0
  j #1
  label #0
  uinc
  label #1
endsub
.jumps abs
sub out.InvertJumps5
  .num_labels 2
  jfep l1
  uinc
l1:
endsub
.jumps sym


sub in.InvertJumps6
  % Second jump never taken, jump disappears
  .num_labels 2
  jtf #0
  jtf #1
  label #0
  uinc
  label #1
endsub
.jumps abs
sub out.InvertJumps6
  .num_labels 2
  uinc
endsub
.jumps sym


sub in.InvertJumps7
  % jdz does not allow this optimisation
  .num_labels 2
  jt #0
  jdz #1
  label #0
  uinc
  label #1
endsub
.jumps abs
sub out.InvertJumps7
  .num_labels 2
  jt l0
  jdz l1
l0:
  uinc
l1:
endsub
.jumps sym



%%
%%  ThreadJumps
%%

sub in.ThreadJumps1
  % just a little hopping around
  .num_labels 4
  jfe #0
  uinc
  j #1
  label #2
  udec
  j #3
  label #0
  label #1
  j #2
  label #3
endsub
.jumps abs
sub out.ThreadJumps1
  .num_labels 4
  jfe l2
  uinc
l2:
  udec
endsub
.jumps sym


sub in.ThreadJumps2
  % a convoluted infinite loop
  .num_labels 5
  label #0
  j #2
  label #1
  j #3
  label #2
  j #4
  label #3
  j #0
  label #4
  j #1
endsub
.jumps abs
sub out.ThreadJumps2
  .num_labels 5
l0:
  j l0
endsub
.jumps sym


sub in.ThreadJumps3
  % another convoluted infinite loop
  .num_labels 5
  label #0
  j #3
  label #1
  j #4
  label #2
  j #0
  label #3
  j #1
  label #4
  j #2
endsub
.jumps abs
sub out.ThreadJumps3
  .num_labels 5
l0:
  j l0
endsub
.jumps sym


sub in.ThreadJumps4
  % jump into an infinite loop
  .num_labels 1
  jt #0
  uinc
  label #0
  j #0
endsub
.jumps abs
sub out.ThreadJumps4
  .num_labels 1
  jt l0
  uinc
l0:
  j l0
endsub
.jumps sym


sub in.ThreadJumps5
  % jump back here, or direct jump to next instruction
  .num_labels 3
  j #0
  label #1
  uinc
  j #2
  label #0
  j #1
  label #2
endsub
sub out.ThreadJumps5
  .num_labels 3
  uinc
endsub


sub in.ThreadJumps6
  % same thing, with a conditional jump
  .num_labels 3
  jfe #0
  label #1
  uinc
  j #2
  label #0
  j #1
  label #2
endsub
sub out.ThreadJumps6
  .num_labels 3
  uinc
endsub


sub in.ThreadJumps7
  % same thing, with a popping conditional jump
  .num_labels 3
  jfep #0
  label #1
  uinc
  j #2
  label #0
  j #1
  label #2
endsub
sub out.ThreadJumps7
  .num_labels 3
  drop 1
  uinc
endsub


sub in.ThreadJumps8
  % jump to conditional jump (not optimized)
  .num_labels 3
  j #0
  label #1
  uinc
  j #2
  label #0
  jfe #1
  label #2
endsub
.jumps abs
sub out.ThreadJumps8
  .num_labels 3
  j l0
l1:
  uinc
  j l2
l0:
  jfe l1
l2:
endsub
.jumps sym


% FIXME: 'catch' to a jump isn't yet optimized
% sub in.ThreadJumps9
%   % catch
%   catch handler
%   uinc
% handler:
%   j skip
% again:
%   uinc
% skip:
%   uinc
%   j again
% endsub
% .jumps abs
% sub out.ThreadJumps9
%   catch skip
%   uinc
% handler:
%   j skip
% again:
%   uinc
% skip:
%   uinc
%   j again
% endsub
% .jumps sym


% FIXME: 'jdz' to a jump isn't yet optimized
% sub in.ThreadJumps10
%   % jdz
%   jdz handler
%   uinc
% handler:
%   j skip
% again:
%   uinc
% skip:
%   uinc
%   j again
% endsub
% .jumps abs
% sub out.ThreadJumps10
%   jdz skip
%   uinc
% handler:
%   j skip
% again:
%   uinc
% skip:
%   uinc
%   j again
% endsub
% .jumps sym


%%
%%  RemoveUnused
%%
%%  j, sthrow, sterminate, sreturn
%%

sub in.RemoveUnused1
  % Construct disappears completely
  j skip
  uinc
  uinc
skip:
  uinc
endsub
sub out.RemoveUnused1
  uinc
endsub


sub in.RemoveUnused2
  % Skipping a piece of code
  j skip
  uinc
  uinc
entry:
  udec
skip:
  uinc
  j entry
endsub
.jumps abs
sub out.RemoveUnused2
  j skip
entry:
  udec
skip:
  uinc
  j entry
endsub
.jumps sym


sub in.RemoveUnused3
  % Other instructions
  jdz a
  jdz b
  jdz c
  jdz d
a:
  sterminate
  uinc
b:
  sthrow
  uinc
c:
  sreturn 0
  uinc
d:
  udec
endsub
.jumps abs
sub out.RemoveUnused3
  jdz a
  jdz b
  jdz c
  jdz d
a:
  sterminate
b:
  sthrow
c:
  sreturn 0
d:
  udec
endsub
.jumps sym


%%
%%  MergeNegation
%%
%%  unot/ubool/upos/uneg/uzap/uisempty/unot2/uinc/udec
%%

sub in.MergeNegation1
  % 'not'
  unot
  unot
    sprint
  unot
  ubool
    sprint
  unot
  upos
    sprint
  unot
  uneg
    sprint
  unot
  uzap
    sprint
  unot
  uisempty
    sprint
  unot
  unot2
    sprint
  unot
  uinc
    sprint
  unot
  udec
endsub
sub out.MergeNegation1
  ubool
    sprint
  unot
    sprint
  unot
  upos
    sprint
  unot
  uneg
    sprint
  unot
  uzap
    sprint
  uisempty
    sprint
  unot
  unot2
    sprint
  unot
  uinc
    sprint
  unot
  udec
endsub


sub in.MergeNegation2
  % 'bool'
  ubool
  unot
    sprint
  ubool
  ubool
    sprint
  ubool
  upos
    sprint
  ubool
  uneg
    sprint
  ubool
  uzap
    sprint
  ubool
  uisempty
    sprint
  ubool
  unot2
    sprint
  ubool
  uinc
    sprint
  ubool
  udec
endsub
sub out.MergeNegation2
  unot
    sprint
  ubool
    sprint
  ubool
  upos
    sprint
  ubool
  uneg
    sprint
  uzap
  ubool
    sprint
  uisempty
    sprint
  unot2
    sprint
  ubool
  uinc
    sprint
  ubool
  udec
endsub


sub in.MergeNegation3
  % 'pos'
  upos
  unot
    sprint
  upos
  ubool
    sprint
  upos
  upos
    sprint
  upos
  uneg
    sprint
  upos
  uzap
    sprint
  upos
  uisempty
    sprint
  upos
  unot2
    sprint
  upos
  uinc
    sprint
  upos
  udec
endsub
sub out.MergeNegation3
  % 'pos'
  upos
  unot
    sprint
  upos
  ubool
    sprint
  upos
    sprint
  uneg
    sprint
  upos
  uzap
    sprint
  upos
  uisempty
    sprint
  upos
  unot2
    sprint
  uinc
    sprint
  udec
endsub


sub in.MergeNegation4
  % 'neg'
  uneg
  unot
    sprint
  uneg
  ubool
    sprint
  uneg
  upos
    sprint
  uneg
  uneg
    sprint
  uneg
  uzap
    sprint
  uneg
  uisempty
    sprint
  uneg
  unot2
    sprint
  uneg
  uinc
    sprint
  uneg
  udec
endsub
sub out.MergeNegation4
  % 'neg'
  uneg
  unot
    sprint
  uneg
  ubool
    sprint
  uneg
    sprint
  upos
    sprint
  uneg
  uzap
    sprint
  uneg
  uisempty
    sprint
  uneg
  unot2
    sprint
  uneg
  uinc
    sprint
  uneg
  udec
endsub


sub in.MergeNegation5
  % 'zap'
  uzap
  unot
    sprint
  uzap
  ubool
    sprint
  uzap
  upos
    sprint
  uzap
  uneg
    sprint
  uzap
  uzap
    sprint
  uzap
  uisempty
    sprint
  uzap
  unot2
    sprint
  uzap
  uinc
    sprint
  uzap
  udec
endsub
sub out.MergeNegation5
  % 'zap'
  uzap
  unot
    sprint
  uzap
  ubool
    sprint
  uzap
  upos
    sprint
  uzap
  uneg
    sprint
  uzap
    sprint
  unot2
    sprint
  unot2
    sprint
  uzap
  uinc
    sprint
  uzap
  udec
endsub


sub in.MergeNegation6
  % 'isempty'
  uisempty
  unot
    sprint
  uisempty
  ubool
    sprint
  uisempty
  upos
    sprint
  uisempty
  uneg
    sprint
  uisempty
  uzap
    sprint
  uisempty
  uisempty
    sprint
  uisempty
  unot2
    sprint
  uisempty
  uinc
    sprint
  uisempty
  udec
endsub
sub out.MergeNegation6
  % 'isempty'
  uisempty
  unot
    sprint
  uisempty
    sprint
  uisempty
  upos
    sprint
  uisempty
  uneg
    sprint
  uisempty
  uzap
    sprint
  drop 1
  pushbool 0
    sprint
  uisempty
  unot2
    sprint
  uisempty
  uinc
    sprint
  uisempty
  udec
endsub


sub in.MergeNegation7
  % 'not2'
  unot2
  unot
    sprint
  unot2
  ubool
    sprint
  unot2
  upos
    sprint
  unot2
  uneg
    sprint
  unot2
  uzap
    sprint
  unot2
  uisempty
    sprint
  unot2
  unot2
    sprint
  unot2
  uinc
    sprint
  unot2
  udec
endsub
sub out.MergeNegation7
  % 'not2'
  unot2
  unot
    sprint
  unot2
    sprint
  unot2
  upos
    sprint
  unot2
  uneg
    sprint
  unot2
  uzap
    sprint
  drop 1
  pushbool 0
    sprint
  unot2
  unot2
    sprint
  unot2
  uinc
    sprint
  unot2
  udec
endsub


sub in.MergeNegation8
  % 'inc'
  uinc
  unot
    sprint
  uinc
  ubool
    sprint
  uinc
  upos
    sprint
  uinc
  uneg
    sprint
  uinc
  uzap
    sprint
  uinc
  uisempty
    sprint
  uinc
  unot2
    sprint
  uinc
  uinc
    sprint
  uinc
  udec
endsub
sub out.MergeNegation8
  % 'inc'
  uinc
  unot
    sprint
  uinc
  ubool
    sprint
  uinc
    sprint
  uinc
  uneg
    sprint
  uinc
  uzap
    sprint
  uinc
  uisempty
    sprint
  uinc
  unot2
    sprint
  uinc
  uinc
    sprint
  upos
endsub


sub in.MergeNegation9
  % 'dec'
  udec
  unot
    sprint
  udec
  ubool
    sprint
  udec
  upos
    sprint
  udec
  uneg
    sprint
  udec
  uzap
    sprint
  udec
  uisempty
    sprint
  udec
  unot2
    sprint
  udec
  uinc
    sprint
  udec
  udec
endsub
sub out.MergeNegation9
  % 'dec'
  udec
  unot
    sprint
  udec
  ubool
    sprint
  udec
    sprint
  udec
  uneg
    sprint
  udec
  uzap
    sprint
  udec
  uisempty
    sprint
  udec
  unot2
    sprint
  upos
    sprint
  udec
  udec
endsub


%%
%%  UnaryCondition
%%
%%  uisempty/unot/uzap/unot2/ubool + popping jump
%%

sub in.UnaryCondition1
  uisempty
  jep ise
    sprint
  uisempty
  jfp isf
    sprint
  uisempty
  jtp ist
    sprint
ise:
  uinc
isf:
  udec
ist:
endsub
.jumps abs
sub out.UnaryCondition1
  drop 1
    sprint
  jtfp isf
    sprint
  jep ist
    sprint
ise:
  uinc
isf:
  udec
ist:
endsub
.jumps sym


sub in.UnaryCondition2
  unot
  jep ise
    sprint
  unot
  jfp isf
    sprint
  unot
  jtp ist
    sprint
ise:
  uinc
isf:
  udec
ist:
endsub
.jumps abs
sub out.UnaryCondition2
  jep ise
    sprint
  jtp isf
    sprint
  jfp ist
    sprint
ise:
  uinc
isf:
  udec
ist:
endsub
.jumps sym


sub in.UnaryCondition3
  uzap
  jep ise
    sprint
  uzap
  jfp isf
    sprint
  uzap
  jtp ist
    sprint
ise:
  uinc
isf:
  udec
ist:
endsub
.jumps abs
sub out.UnaryCondition3
  jfep ise
    sprint
  drop 1
    sprint
  jtp ist
    sprint
ise:
  upos        % isf is not references, thus uneg/upos are merged
ist:
endsub
.jumps sym


sub in.UnaryCondition4
  unot2
  jep ise
    sprint
  unot2
  jfp isf
    sprint
  unot2
  jtp ist
    sprint
ise:
  uinc
isf:
  udec
ist:
endsub
.jumps abs
sub out.UnaryCondition4
  drop 1
    sprint
  jtp isf
    sprint
  jfep ist
    sprint
ise:
  uinc
isf:
  udec
ist:
endsub
.jumps sym


sub in.UnaryCondition5
  ubool
  jep ise
    sprint
  ubool
  jfp isf
    sprint
  ubool
  jtp ist
    sprint
ise:
  uinc
isf:
  udec
ist:
endsub
.jumps abs
sub out.UnaryCondition5
  jep ise
    sprint
  jfp isf
    sprint
  jtp ist
    sprint
ise:
  uinc
isf:
  udec
ist:
endsub
.jumps sym


%%
%%  FoldUnary
%%
%%  uzap/uneg/upos/unot/unot2/ubool/uabs/uisempty/uisstring/uisnum/utrunc/uround/uinc/udec/ubitnot
%%  after pushint/pushbool
%%

sub in.FoldUnary1
  % Various 'zap's
  pushint 0
  uzap
  pushint 1
  uzap
  pushint 2
  uzap
  pushint -1
  uzap
  pushbool 0
  uzap
  pushbool 1
  uzap
  pushbool -1
  uzap
endsub
sub out.FoldUnary1
  pushbool -1
  pushint 1
  pushint 2
  pushint -1
  pushbool -1
  pushbool 1
  pushbool -1
endsub


sub in.FoldUnary2
  % Various 'neg's
  pushint 0
  uneg
  pushint 1
  uneg
  pushint 2
  uneg
  pushint -1
  uneg
  pushbool 0
  uneg
  pushbool 1
  uneg
  pushbool -1
  uneg
endsub
sub out.FoldUnary2
  pushint 0
  pushint -1
  pushint -2
  pushint 1
  pushint 0
  pushint -1
  pushbool -1
endsub


sub in.FoldUnary3
  % All instructions
  pushint 10
  uzap
  pushint 10
  uneg
  pushint 10
  upos
  pushint 10
  unot
  pushint 10
  unot2
  pushint 10
  ubool
  pushint 10
  uabs
  pushint 10
  uisempty
  pushint 10
  uisstr
  pushint 10
  uisnum
  pushint 10
  utrunc
  pushint 10
  uround
  pushint 10
  uinc
  pushint 10
  udec
  pushint 10
  ubitnot
endsub
sub out.FoldUnary3
  pushint 10
  pushint -10
  pushint 10
  pushbool 0
  pushbool 0
  pushbool 1
  pushint 10
  pushbool 0
  pushbool 0
  pushbool 1
  pushint 10
  pushint 10
  pushint 11
  pushint 9
  pushint -11
endsub


sub in.FoldUnary4
  % Boundary cases
  pushint 32767
  uinc
  pushint 32767
  udec
  pushint -32768
  uinc
  pushint -32768
  udec
endsub
sub out.FoldUnary4
  pushint 32767
  uinc
  pushint 32766
  pushint -32767
  pushint -32768
  udec
endsub


%%
%%  FoldBinaryInt
%%
%%  badd/bsub/mul/div/idiv/pow x -1/0/+1
%%

sub in.FoldBinaryInt1
  pushint 1
  badd
    sprint
  pushint -1
  badd
    sprint
  pushint 0
  badd
endsub
sub out.FoldBinaryInt1
  uinc
    sprint
  udec
    sprint
  upos
endsub


sub in.FoldBinaryInt2
  pushint 1
  bsub
    sprint
  pushint -1
  bsub
    sprint
  pushint 0
  bsub
endsub
sub out.FoldBinaryInt2
  udec
    sprint
  uinc
    sprint
  upos
endsub


sub in.FoldBinaryInt3
  pushint 1
  bmul
    sprint
  pushint -1
  bmul
    sprint

  pushint 1
  bdiv
    sprint
  pushint -1
  bdiv
    sprint

  pushint 1
  bidiv
    sprint
  pushint -1
  bidiv
endsub
sub out.FoldBinaryInt3
  upos
    sprint
  uneg
    sprint
  upos
    sprint
  uneg
    sprint
  upos
    sprint
  uneg
endsub


sub in.FoldBinaryInt4
  pushint 1
  bpow
endsub
sub out.FoldBinaryInt4
  upos
endsub


%%
%%  FoldJump
%%

sub in.FoldJump1
  % condition is true
  jdz a
  jdz b
  jdz c
a:
  uinc
  pushint 1
  jtp doit
b:
  udec
  pushint 1
  jfp doit
c:
  uneg
  pushint 1
  jep doit
  uinc
doit:
endsub
.jumps abs
sub out.FoldJump1
  jdz a
  jdz b
  jdz c
a:
  uinc
  j doit
b:
  udec
c:
  uneg
  uinc
doit:
endsub
.jumps sym


sub in.FoldJump2
  % condition is false
  jdz a
  jdz b
  jdz c
a:
  uinc
  pushint 0
  jtp doit
b:
  udec
  pushint 0
  jfp doit
c:
  uneg
  pushint 0
  jep doit
  uinc
doit:
endsub
.jumps abs
sub out.FoldJump2
  jdz a
  jdz b
  jdz c
a:
  uinc
b:
  udec
  j doit
c:
  uneg
  uinc
doit:
endsub
.jumps sym


sub in.FoldJump3
  % condition is empty
  jdz a
  jdz b
  jdz c
a:
  uinc
  pushbool -1
  jtp doit
b:
  udec
  pushbool -1
  jfp doit
c:
  uneg
  pushbool -1
  jep doit
  uinc
doit:
endsub
.jumps abs
sub out.FoldJump3
  jdz a
  jdz b
  jdz c
a:
  uinc
b:
  udec
c:
  uneg
doit:
endsub
.jumps sym


sub in.FoldJump4
  % jump without pop
  jdz a
  jdz b
  jdz c
a:
  uinc
  pushbool -1
  jt doit
b:
  udec
  pushbool -1
  jf doit
c:
  uneg
  pushbool -1
  je doit
  uinc
doit:
endsub
.jumps abs
sub out.FoldJump4
  jdz a
  jdz b
  jdz c
a:
  uinc
  pushbool -1
b:
  udec
  pushbool -1
c:
  uneg
  pushbool -1
doit:
endsub
.jumps sym


%%
%%  PopPush
%%

sub in.PopPush1
  .local a
  poploc a
  pushloc a
endsub
sub out.PopPush1
  .local a
  storeloc a
endsub


sub in.PopPush2
  popvar a
  pushvar a
endsub
sub out.PopPush2
  % not optimized, because pop into a property implies a type-cast
  popvar a
  pushvar a
endsub


sub in.PopPush3
  .local a
  poploc a
  pushglob #0
endsub
sub out.PopPush3
  .local a
  % not optimized, different scope
  poploc a
  pushglob #0
endsub


%%
%%  CompareNC
%%
%%  bcmpeqNC, bcmpneNC, bfirststrNC, breststrNC, bfindstrNC
%%

sub in.CompareNC1
  % Check that all instructions are accepted
  pushlit ""
  bcmpeqNC
    sprint
  pushlit ""
  bcmpneNC
    sprint
  pushlit ""
  bfirststrNC
    sprint
  pushlit ""
  breststrNC
    sprint
  pushlit ""
  bfindstrNC
endsub
sub out.CompareNC1
  pushlit(b) ""
  bcmpeq
    sprint
  pushlit(b) ""
  bcmpne
    sprint
  pushlit(b) ""
  bfirststr
    sprint
  pushlit(b) ""
  breststr
    sprint
  pushlit(b) ""
  bfindstr
endsub


sub in.CompareNC2
  % Check instructions that are not accepted
  pushlit "["
  bcmpgeNC
    sprint
  pushlit "["
  bcmpgtNC
    sprint
  pushlit "["
  bcmpleNC
    sprint
  pushlit "["
  bcmpltNC
endsub
sub out.CompareNC2
  pushlit(b) "["
  bcmpgeNC
    sprint
  pushlit(b) "["
  bcmpgtNC
    sprint
  pushlit(b) "["
  bcmpleNC
    sprint
  pushlit(b) "["
  bcmpltNC
endsub


sub in.CompareNC3
  % Check literals that are accepted
  pushint 1
  bcmpneNC
    sprint
  pushbool 1
  bcmpneNC
    sprint
  pushlit 1
  bcmpneNC
    sprint
  pushlit 1.0
  bcmpneNC
    sprint
  pushlit ""
  bcmpneNC
    sprint
  pushlit "."
  bcmpneNC
    sprint
  pushlit "["
  bcmpneNC
    sprint
  pushlit "}"
  bcmpneNC
endsub
sub out.CompareNC3
  pushint 1
  bcmpne
    sprint
  pushbool 1
  bcmpne
    sprint
  pushlit(b) 1
  bcmpne
    sprint
  pushlit(b) 1.0
  bcmpne
    sprint
  pushlit(b) ""
  bcmpne
    sprint
  pushlit(b) "."
  bcmpne
    sprint
  pushlit(b) "["
  bcmpne
    sprint
  pushlit(b) "}"
  bcmpne
endsub


sub in.CompareNC4
  % Check arguments that are not accepted
  pushvar A
  bcmpneNC
    sprint
  pushlit "A"
  bcmpneNC
    sprint
  pushlit "a"
  bcmpneNC
    sprint
  pushlit "......a......"
  bcmpneNC
endsub
sub out.CompareNC4
  pushvar A
  bcmpneNC
    sprint
  pushlit(b) "A"
  bcmpneNC
    sprint
  pushlit(b) "a"
  bcmpneNC
    sprint
  pushlit(b) "......a......"
  bcmpneNC
endsub


%%
%%  Entry Point
%%

Sub main
  .defsubs
EndSub

