/**
  *  \file ui/widgets/cardtab.h
  *  \brief UICardGroupTab widget
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UI_WIDGETS_CARDTAB_H
#define PCC_V2_UI_WIDGETS_CARDTAB_H

#include "cpluslib/ptrvector.h"
#include "cpluslib/sigslot.h"
#include "cpluslib/string.h"
#include "ui/keystring.h"
#include "ui/ui.h"

class UICardGroup;

/** \class UICardGroupTab
    \brief Tab bar for a card group

    This provides a bar of clickable register tabs to control the
    focused widget of a UICardGroup. Users can click at the tabs, hit
    the tab hot-key, or use Tab/Shift-Tab to navigate through all
    pages.

    You must build up the tabs using addTab(). */
class UICardGroupTab : public UIWidget {
    struct TabInfo;

    ptr_vector<TabInfo>   tabs;
    UICardGroup&          group;
    SignalConnection      group_conn;
    int keys;

 public:
    enum {
        UsePageKeys = 1,
        UseTabKeys = 2
    };

    UICardGroupTab(UICardGroup& group);
    ~UICardGroupTab();

    void    drawContent(GfxCanvas& can);
    void    getLayoutInfo(LayoutInfo& li);
    bool    handleEvent(const UIEvent& e, bool second_pass);

    void    addTab(int id, const string_t& name, UIKey key);
    void    addTab(int id, const UIKeyString& key);

    int     getCurrentTab() const;
    bool    setCurrentTab(int);

    int     getCurrentTabIndex() const;
    bool    setCurrentTabIndex(int n);

    int     getNumTabs() const { return tabs.size(); }

    void    setKeys(int keys) { this->keys = keys; }
};

#endif
