/**
  *  \file ui/widgets/timer.cc
  *  \brief Timer Widget
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "ui/widgets/timer.h"

static inline int
convertTime(int msecs)
{
    return (msecs + TIMER_MS-1) / TIMER_MS;
}

/******************************** UITimer ********************************/

/** Constructor. The timer will be idle (not running). */
UITimer::UITimer()
    : this_timer(0),
      next_timer(0),
      counter(0)
{ }

/** Destructor. */
UITimer::~UITimer()
{ }

/** Start single-shot timer. The timer will fire once after \c msecs milliseconds.
    The counter (getCounter()) will start at 0. */
void
UITimer::startSingleShot(int msecs)
{
    this_timer = convertTime(msecs);
    next_timer = 0;
    counter    = 0;
}

/** Start periodic timer. The timer will fire every \c msecs milliseconds.
    The counter (getCounter()) will start at 0. */
void
UITimer::startPeriodic(int msecs)
{
    this_timer = convertTime(msecs);
    next_timer = this_timer;
    counter    = 0;
}

bool
UITimer::handleEvent(const UIEvent& event, bool second_pass)
{
    if (event.type == ev_Tick && !second_pass) {
        if (this_timer != 0) {
            --this_timer;
            if (this_timer == 0) {
                ++counter;
                this_timer = next_timer;
                sig_tick.raise();
            }
        }
    }
    return UIWidget::handleEvent(event, second_pass);
}

void
UITimer::getLayoutInfo(LayoutInfo& li)
{
    li.min_size = li.pref_size = GfxPoint(0, 0);
    li.flags = li_NoLayout;
}

void
UITimer::drawContent(GfxCanvas& /*can*/)
{ }
